@@ -1,7 +1,7 @@
 /*
    sslh-fork: forking server
 
-# Copyright (C) 2007-2012  Yves Rutschle
+# Copyright (C) 2007-2021  Yves Rutschle
 # 
 # This program is free software; you can redistribute it
 # and/or modify it under the terms of the GNU General Public
@@ -22,6 +22,9 @@
 
 #include "common.h"
 #include "probe.h"
+#include "sslh-conf.h"
+#include "tcp-probe.h"
+#include "log.h"
 
 #ifdef LIBBSD
 #include <bsd/unistd.h>
@@ -29,8 +32,6 @@
 
 const char* server_type = "sslh-fork";
 
-#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
-
 /* shovels data from one fd to the other and vice-versa 
    returns after one socket closed
  */
@@ -57,9 +58,8 @@ int shovel(struct connection *cnx)
       for (i = 0; i < 2; i++) {
           if (FD_ISSET(cnx->q[i].fd, &fds)) {
               res = fd2fd(&cnx->q[1-i], &cnx->q[i]);
-              if (!res) {
-                  if (cfg.verbose) 
-                      fprintf(stderr, "%s %s", i ? "client" : "server", "socket closed\n");
+              if (res == FD_CNXCLOSED) {
+                  print_message(msg_fd, "%s %s", i ? "client" : "server", "socket closed\n");
                   return res;
               }
           }
@@ -99,8 +99,7 @@ void start_shoveler(int in_socket)
        } else {
            /* Timed out: it's necessarily SSH */
            cnx.proto = timeout_protocol();
-           if (cfg.verbose) 
-               log_message(LOG_INFO, "timed out, connect to %s\n", cnx.proto->name);
+           print_message(msg_fd, "timed out, connect to %s\n", cnx.proto->name);
            break;
        }
    }
@@ -111,9 +110,11 @@ void start_shoveler(int in_socket)
    }
 
    /* Connect the target socket */
-   out_socket = connect_addr(&cnx, in_socket);
+   out_socket = connect_addr(&cnx, in_socket, BLOCKING);
    CHECK_RES_DIE(out_socket, "connect");
 
+   set_capabilities(0);
+
    cnx.q[1].fd = out_socket;
 
    get_connection_desc(&desc, &cnx);
@@ -127,13 +128,12 @@ void start_shoveler(int in_socket)
    close(in_socket);
    close(out_socket);
    
-   if (cfg.verbose)
-      fprintf(stderr, "connection closed down\n");
+   print_message(msg_fd, "connection closed down\n");
 
    exit(0);
 }
 
-static int *listener_pid;
+static pid_t *listener_pid;
 static int listener_pid_number = 0;
 
 void stop_listeners(int sig)
@@ -145,7 +145,7 @@ void stop_listeners(int sig)
     }
 }
 
-void set_listen_procname(int listen_socket)
+void set_listen_procname(struct listen_endpoint *listen_socket)
 {
 #ifdef LIBBSD
     int res;
@@ -155,7 +155,7 @@ void set_listen_procname(int listen_socket)
 
     addr.ai_addr = (struct sockaddr*)&ss;
     addr.ai_addrlen = sizeof(ss);
-    res = getsockname(listen_socket, addr.ai_addr, &addr.ai_addrlen);
+    res = getsockname(listen_socket->socketfd, addr.ai_addr, &addr.ai_addrlen);
     if (res != -1) {
         sprintaddr(listen_addr, sizeof(listen_addr), &addr);
         setproctitle("listener %s", listen_addr);
@@ -163,50 +163,70 @@ void set_listen_procname(int listen_socket)
 #endif
 }
 
-void main_loop(int listen_sockets[], int num_addr_listen)
+
+/* TCP listener: connections, fork a child for each new connection 
+ * IN: 
+ *      endpoint: array of listening endpoint objects
+ *      num_endpoints: size of endpoint array
+ *      active_endpoint: which endpoint is this listener working on
+ * Does not return
+ * */
+void tcp_listener(struct listen_endpoint* endpoint, int num_endpoints, int active_endpoint)
 {
-    int in_socket, i, res;
+    int i, in_socket;
+
+    while (1) {
+        in_socket = accept(endpoint[active_endpoint].socketfd, 0, 0);
+        CHECK_RES_RETURN(in_socket, "accept", /*void*/ );
+        print_message(msg_fd, "accepted fd %d\n", in_socket);
+
+        switch(fork()) {
+        case -1: print_message(msg_system_error, "fork failed: err %d: %s\n", errno, strerror(errno));
+                 break;
+
+        case 0: /* In child process */
+                 /* Shoveler processes don't need to hog file descriptors */
+                 for (i = 0; i < num_endpoints; i++)
+                     close(endpoint[i].socketfd);
+                 start_shoveler(in_socket);
+                 exit(0);
+
+        default: /* In parent process */
+                 break;
+        }
+        close(in_socket);
+    }
+}
+
+
+void main_loop(struct listen_endpoint listen_sockets[], int num_addr_listen)
+{
+    int i, res;
     struct sigaction action;
 
     listener_pid_number = num_addr_listen;
     listener_pid = malloc(listener_pid_number * sizeof(listener_pid[0]));
     CHECK_ALLOC(listener_pid, "malloc");
 
+    tcp_init();
+
     /* Start one process for each listening address */
     for (i = 0; i < num_addr_listen; i++) {
         listener_pid[i] = fork();
         switch(listener_pid[i]) {
-        // Log if fork() fails for some reason
-        case -1: log_message(LOG_ERR, "fork failed: err %d: %s\n", errno, strerror(errno));
+        /* Log if fork() fails for some reason */
+        case -1: print_message(msg_system_error, "fork failed: err %d: %s\n", errno, strerror(errno));
                  break;
-        // We're in the child, we have work to do 
+        /* We're in the child, we have work to do  */
         case 0:
-            /* Listening process just accepts a connection, forks, and goes
-             * back to listening */
-            set_listen_procname(listen_sockets[i]);
-            while (1)
-            {
-                in_socket = accept(listen_sockets[i], 0, 0);
-                if (cfg.verbose) fprintf(stderr, "accepted fd %d\n", in_socket);
-
-                switch(fork()) {
-                case -1: log_message(LOG_ERR, "fork failed: err %d: %s\n", errno, strerror(errno));
-                         break;
-
-                /* In child process */
-                case 0:
-                    for (i = 0; i < num_addr_listen; ++i)
-                        close(listen_sockets[i]);
-                    start_shoveler(in_socket);
-                    exit(0);
-
-                /* In parent process */
-                default: break;
-                }
-                close(in_socket);
-            }
+            set_listen_procname(&listen_sockets[i]);
+            if (listen_sockets[i].type == SOCK_DGRAM)
+                print_message(msg_config_error, "UDP not (yet?) supported in sslh-fork\n");
+            else
+                tcp_listener(listen_sockets, num_addr_listen, i);
 	    break;
-	// We're in the parent, we don't need to do anything
+
+	/* We're in the parent, we don't need to do anything */
 	default:
 	    break;
         }
@@ -220,6 +240,7 @@ void main_loop(int listen_sockets[], int num_addr_listen)
     res = sigaction(SIGTERM, &action, NULL);
     CHECK_RES_DIE(res, "sigaction");
 
+
     wait(NULL);
 }
 
